function inspectZod(schema, path = []) {
    if ('typeName' in schema._def) {
        debug('  '.repeat(path.length), path.at(-1) ?? '<root>', '::', schema._def.typeName)
        if (schema._def.typeName === 'ZodUnion') {
            schema._def.options.forEach(option => inspectZod(option, [...path, '<union>']))
            return
        }
        if (schema._def.typeName === 'ZodObject') {
            Object.entries(schema._def.shape()).forEach(([k, v]) => {
                inspectZod(v, [...path, k])
            })
            return
        }
        if (schema._def.typeName === 'ZodTuple') {
            schema._def.items.forEach((item, i) => {
                inspectZod(item, [...path, i])
            })

            return
        }
        if (schema._def.typeName === 'ZodArray') {
            inspectZod(schema._def.type, [...path, '<index>'])

            return
        }
        if (schema._def.typeName === 'ZodOptional') {
            inspectZod(schema._def.innerType, [...path, '?'])

            return
        }
        if (schema._def.typeName === 'ZodLiteral') {
            debug('  '.repeat(path.length + 1), schema._def)

            return
        }
        if (schema._def.typeName === 'ZodString') {
            return
        }
        if (schema._def.typeName === 'ZodRecord') {
            inspectZod(schema._def.keyType, [...path, '<key>'])
            inspectZod(schema._def.valueType, [...path, '<value>'])
            return
        }
        debug('_def:', schema._def)
    }
}

const ZodField = ({ value, setValue, schema, path }) => {
    if (schema._def.typeName === 'ZodString') {
        return (
            <div>
                <label>{path.at(-1)}</label>
                <input value={value} onInput={e => setValue(path, e.target.value)} />
            </div>
        )
    }
    if (schema._def.typeName === 'ZodNumber') {
        return (
            <div>
                <label>{path.at(-1)}</label>
                <input value={value} onInput={e => setValue(path, e.target.value)} />
            </div>
        )
    }
    if (schema._def.typeName === 'ZodBoolean') {
        return (
            <div>
                <label>{path.at(-1)}</label>
                <input
                    type="checkbox"
                    checked={value}
                    onInput={e => setValue(path, e.target.checked)}
                />
            </div>
        )
    }
    if (schema._def.typeName === 'ZodDate') {
        return (
            <div>
                <label>{path.at(-1)}</label>
                <input type="date" value={value} onInput={e => setValue(path, e.target.value)} />
            </div>
        )
    }
    if (schema._def.typeName === 'ZodObject') {
        return (
            <div>
                {Object.entries(schema._def.shape()).map(([k, v]) => {
                    return (
                        <ZodField
                            value={value[k]}
                            setValue={setValue}
                            schema={v}
                            path={[...path, k]}
                        />
                    )
                })}
            </div>
        )
    }
    if (schema._def.typeName === 'ZodArray') {
        return (
            <div>
                {value.map((v, i) => {
                    return (
                        <ZodField
                            value={v}
                            setValue={setValue}
                            schema={schema._def.type}
                            path={[...path, i]}
                        />
                    )
                })}
                <button onClick={() => setValue(path, [...value, ''])}>Add</button>
            </div>
        )
    }
    if (schema._def.typeName === 'ZodOptional') {
        return (
            <div>
                <ZodField
                    value={value}
                    setValue={setValue}
                    schema={schema._def.innerType}
                    path={path}
                />
            </div>
        )
    }
    if (schema._def.typeName === 'ZodLiteral') {
        return (
            <div>
                <ZodField value={value} setValue={setValue} schema={schema} path={path} />
            </div>
        )
    }
    if (schema._def.typeName === 'ZodUnion') {
        return (
            <div>
                <select value={value} onInput={e => setValue(path, e.target.value)}>
                    {schema._def.options.map(option => {
                        return <option value={option._def.value}>{option._def.value}</option>
                    })}
                </select>
            </div>
        )
    }
}

const ZodObject = ({ value, setValue, schema, path }) => {
    return (
        <div>
            {Object.entries(schema._def.shape()).map(([k, v]) => {
                return (
                    <ZodField value={value[k]} setValue={setValue} schema={v} path={[...path, k]} />
                )
            })}
        </div>
    )
}

const ZodArray = ({ value, setValue, schema, path }) => {
    return (
        <div>
            {value.map((v, i) => {
                return (
                    <ZodField
                        value={v}
                        setValue={setValue}
                        schema={schema._def.type}
                        path={[...path, i]}
                    />
                )
            })}
            <button onClick={() => setValue(path, [...value, ''])}>Add</button>
        </div>
    )
}

const ZodOptional = ({ value, setValue, schema, path }) => {
    return (
        <div>
            <ZodField
                value={value}
                setValue={setValue}
                schema={schema._def.innerType}
                path={path}
            />
        </div>
    )
}

const ZodLiteral = ({ value, setValue, schema, path }) => {
    return (
        <div>
            <ZodField value={value} setValue={setValue} schema={schema} path={path} />
        </div>
    )
}

const ZodUnion = ({ value, setValue, schema, path }) => {
    return (
        <div>
            <select value={value} onInput={e => setValue(path, e.target.value)}>
                {schema._def.options.map(option => {
                    return <option value={option._def.value}>{option._def.value}</option>
                })}
            </select>
        </div>
    )
}

export const ZodForm = ({ value, setValue, schema }) => {
    return (
        <div>
            {schema._def.typeName === 'ZodObject' && (
                <ZodObject value={value} setValue={setValue} schema={schema} path={[]} />
            )}
            {schema._def.typeName === 'ZodArray' && (
                <ZodArray value={value} setValue={setValue} schema={schema} path={[]} />
            )}
            {schema._def.typeName === 'ZodOptional' && (
                <ZodOptional value={value} setValue={setValue} schema={schema} path={[]} />
            )}
            {schema._def.typeName === 'ZodLiteral' && (
                <ZodLiteral value={value} setValue={setValue} schema={schema} path={[]} />
            )}
            {schema._def.typeName === 'ZodUnion' && (
                <ZodUnion value={value} setValue={setValue} schema={schema} path={[]} />
            )}
        </div>
    )
}
