import { access } from 'fs/promises'

export const normalizeURL = (url: string) => {
    if (!url.startsWith('/') && !url.startsWith('https://') && !url.startsWith('http://')) {
        url = `https://${url}`
    }

    return url
}

export const clsx = (...args: any[]) =>
    args
        .filter(Boolean)
        .flatMap(s => (typeof s === 'string' ? s.split(' ') : [s]))
        .join(' ')

/**
 * Modern alternative to fs.existsSync
 */
export async function exists(path: string) {
    try {
        await access(path)
        return true
    } catch (err) {
        // @ts-ignore
        if (err.code === 'ENOENT') {
            return false
        } else {
            throw err
        }
    }
}

export function sleep(timeout: number) {
    return new Promise(resolve => {
        setTimeout(resolve, timeout)
    })
}

export class JsonResponse extends Response {
    constructor(data: any) {
        super(JSON.stringify(data), {
            status: 200,
            headers: {
                'Content-Type': 'application/json',
            },
        })
    }
}

export class JsonStreamResponse extends Response {
    constructor(create: (send: (data: any) => void) => () => void | undefined) {
        let cancelFn: () => void | undefined

        const stream = new ReadableStream({
            start(controller) {
                const sendEvent = (data: any) => {
                    controller.enqueue(`data: ${JSON.stringify(data)}\r\n\r\n`)
                }

                cancelFn = create(sendEvent)
            },
            cancel() {
                cancelFn?.()
            },
        })

        super(stream, {
            status: 200,
            headers: {
                'Content-Type': 'text/event-stream',
                'Connection': 'keep-alive',
                'Cache-Control': 'no-cache',
            },
        })
    }
}

export function createQueryUrl(base: string, options: Record<string, string>) {
    return `${base}?${Object.entries(options)
        .map(([k, v]) => k + '=' + encodeURIComponent(v))
        .join('&')}`
}
