import { OnJobLog, getJob, getJobLogs } from '@/jobs'
import { JsonResponse, JsonStreamResponse, createQueryUrl } from '@/lib/utils'
import { debug } from '@/logger'
import type { APIRoute } from 'astro'

export const GET: APIRoute = async ({ params: { uuid }, url, redirect }) => {
    const rawLogs = await getJobLogs(uuid!)

    const format = url.searchParams.get('format')
    switch (format) {
        case 'raw':
            return new Response(rawLogs)
        case 'json':
            return new JsonResponse(rawLogs.trim().split('\n'))
        case 'sse':
            return new JsonStreamResponse(sendData => {
                const jobLog = (content: string) => sendData({ type: 'log', content })

                debug('[SSE] Registering job log client')
                OnJobLog.addListener(uuid!, jobLog)

                return () => {
                    // cancel
                    OnJobLog.removeListener(uuid!, jobLog)
                    debug('[SSE] Un-registered job log client')
                }
            })
        default:
            return redirect(
                createQueryUrl('/error', {
                    message: `Invalid format "${format}"`,
                })
            )
    }
}
